/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2010 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.lifecycle;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.julia.control.lifecycle.ChainedIllegalLifeCycleException;
import org.ow2.frascati.tinfi.control.content.ContentInstantiationException;
import org.ow2.frascati.tinfi.control.content.SCAContentController;

/**
 * Mixin layer to be included in the lifecycle controller for managing the
 * EagerInit policy, the Start and the Stop policy.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public abstract class SCALifeCycleMixin {
    
    // -------------------------------------------------------------------------
    // Private constructor
    // -------------------------------------------------------------------------

    private SCALifeCycleMixin() {}

    // -------------------------------------------------------------------------
    // Implementation of the LifeCycleCoordinator interface
    // -------------------------------------------------------------------------

    public void setFcContentState( boolean started )
    throws IllegalLifeCycleException {
    	
    	/*
    	 * Do not propagate to super layers. We do not want to execute the code
    	 * defined in ContainerLifeCycleMixin#setFcContentState(boolean).
    	 */
    	
        try {
        	if(started) {
                _this_weaveableSCACC.eagerInit();
                _this_weaveableSCACC.start();
        	}
        	else {
                _this_weaveableSCACC.stop();
        	}
        }
        catch( ContentInstantiationException ie ) {
            throw new ChainedIllegalLifeCycleException(ie,null,"");
        }
    }
    

    // -------------------------------------------------------------------------
    // Fields and methods required by the mixin class in the base class
    // -------------------------------------------------------------------------

    /**
     * The {@link SCAContentController} interface of the component to which
     * this controller object belongs.
     */
    public SCAContentController _this_weaveableSCACC;
}
