/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2010 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.property;

import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.julia.Controller;
import org.objectweb.fractal.julia.InitializationContext;
import org.ow2.frascati.tinfi.control.content.SCAContentController;

/**
 * Implementing the {@link SCAContentController} control interface for
 * scaPrimitive components.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 1.4
 */
public abstract class SCAPrimitivePropertyControllerMixin
implements Controller, SCAPropertyController {
    
    // -------------------------------------------------------------------------
    // Implementation of the Controller interface
    // -------------------------------------------------------------------------

	public void initFcController( InitializationContext ic )
	throws InstantiationException {
		_super_initFcController(ic);
	}

	
    // -------------------------------------------------------------------------
    // Implementation of the SCAPropertyController interface
    // -------------------------------------------------------------------------

    public void setValue( String name, Object value )
    throws IllegalArgumentException {

    	// Re-inject the value on current content instances
        _this_weaveableSCACC.setPropertyValue(name,value);
    }
        
    public boolean containsDeclaredPropertyName( String name ) {
        boolean b = _this_weaveableSCACC.containsPropertyName(name);
        return b;
    }
    
    public String[] getDeclaredPropertyNames() {
        String[] names = _this_weaveableSCACC.getPropertyNames();
        return names;
    }
    
    public Class<?> getDeclaredPropertyType( String name ) {
    	Class<?> type = _this_weaveableSCACC.getPropertyType(name);
    	return type;
    }
    
    
    // -------------------------------------------------------------------------
    // Fields and methods required by the mixin class in the base class
    // -------------------------------------------------------------------------

	protected abstract void _super_initFcController( InitializationContext ic )
	throws InstantiationException;

    private SCAContentController _this_weaveableSCACC;
}
